@@ -15,6 +15,7 @@
 # -------------------------------------------------------------------------- #
 from globus.provision.core.topology import Topology, Node
 import tempfile
+from globus.provision.common.threads import SIGINTWatcher
 
 """
 The CLI: A console frontend to Globus Provision that allows a user to request instances, 
@@ -35,7 +36,7 @@ from globus.provision import RELEASE
 class Command(object):
     """Base class for all Globus Provisioning commands"""
     
-    def __init__(self, argv, root = False):
+    def __init__(self, argv, root = False, disable_sigintwatch = False):
         
         if root:
             if getpass.getuser() != "root":
@@ -65,6 +66,9 @@ class Command(object):
                                          "(instead of the default ~/.globusprovision/instances/)")
         
         colorama.init(autoreset = True)
+        
+        if not disable_sigintwatch:
+            SIGINTWatcher(self.cleanup_after_kill)
                 
 
     def parse_options(self):
@@ -131,7 +135,7 @@ class Command(object):
             return Fore.BLUE + Style.BRIGHT + state_str + reset
         elif state == Node.STATE_RUNNING:
             return Fore.GREEN + Style.BRIGHT + state_str + reset
-        elif state in (Node.STATE_STARTING, Node.STATE_RUNNING_UNCONFIGURED, Node.STATE_CONFIGURING, Node.STATE_STOPPING, Node.STATE_RESUMING, Node.STATE_TERMINATING):
+        elif state in (Node.STATE_STARTING, Node.STATE_RUNNING_UNCONFIGURED, Node.STATE_CONFIGURING, Node.STATE_RECONFIGURING, Node.STATE_STOPPING, Node.STATE_STOPPING_CONFIGURING, Node.STATE_STOPPING_CONFIGURED, Node.STATE_RESUMING, Node.STATE_RESUMED_UNCONFIGURED, Node.STATE_RESUMED_RECONFIGURING, Node.STATE_TERMINATING):
             return Fore.YELLOW + Style.BRIGHT + state_str + reset
         elif state in (Node.STATE_TERMINATED, Node.STATE_FAILED):
             return Fore.RED + Style.BRIGHT + state_str + reset
